<cfcomponent extends="Authenticator" 
   displayname="LDAPAuthenticator" hint="LDAP auth.">
   <!--- your Active Directory Domain --->
   <cfset domain = "mycompany.com">
   <!--- the address of your ldap server --->
   <cfset ldapServer         = "localhost">
   <!--- the base DN to use when querying the server --->
   <cfset ldapBaseDn         = "dc=mycompany,dc=com">
   <!--- the user ldap attribute for comparing against the username --->
   <cfset ldapUidAttr         = "sAMAccountName">
   <!--- the user ldap attribute for finding a user's groups or roles --->
   <cfset ldapRoleAttr       = "memberOf">
   <!--- the group ldap attribute for getting a role's name --->
   <cfset ldapRoleNameAttr      = "name">
   <!--- map ldap user attribute names to expected session variable names --->
   <cfset ldapmap            = StructNew()>
   <cfset ldapmap.displayName   = "displayName">
   <cfset ldapmap.firstName   = "givenName">
   <cfset ldapmap.lastName      = "sn">
   <cfset ldapmap.city         = "l">
   <cfset ldapmap.state      = "st">
   <cfset ldapmap.country      = "c">
   <cfset ldapmap.email      = "mail">
   <cfset ldapmap.phone      = "telephonenumber">
   
   <cffunction name="authenticate" returntype="boolean" output="false">
      <cfargument name="username" type="string" required="true">
      <cfargument name="password" type="string" required="true">
         <cftry>
            <cfset filter="(#ldapUidAttr#=#username#)">
            <cfset attributes="">
            <cfloop list="#StructKeyList(ldapmap)#" index="prop">
               <cfset ldapattr = StructFind(ldapmap,prop)>
               <cfset attributes = ListAppend(attributes,ldapattr)>
            </cfloop>
      
            <!--- attempt to authenticate the user and 
               download his/her profile info --->
            <cfldap server="#ldapServer#" 
               username="#arguments.username#@#domain#"
               password="#arguments.password#"
               action="query"
               start="#ldapBaseDn#"
               filter="#filter#"
               attributes="#attributes#"
               name="results">
      
            <!--- set the user properties --->
            <cfloop list="#StructKeyList(ldapmap)#" index="prop">
               <cfset ldapattr = StructFind(ldapmap,prop)>
               <cfset ldapval  = StructFind(results,ldapattr)>
               <cfset tmp = SetVariable("user.#prop#",ldapval)>
            </cfloop>
      
            <!--- get the user's roles --->
            <cfldap server="#ldapServer#" 
               username="#arguments.username#@#domain#"
               password="#arguments.password#"
               action="query"
               start="#ldapBaseDn#"
               filter="#filter#"
               attributes="#ldapRoleAttr#"
               name="groups">
      
            <cfset roles = ArrayNew(1)>
            <cfset i = 1>
            <cfset grouplist = StructFind(groups,ldapRoleAttr)>
            <cfset grouplist = Replace(grouplist,", ",chr(253),"all")>
            <cfloop list="#grouplist#" index="group" delimiters="#chr(253)#">
               <!--- get the role detail --->
               <cfldap server="#ldapServer#" 
                  username="#arguments.username#@#domain#"
                  password="#arguments.password#"
                  action="query"
                  start="#group#"
                  filter="(objectClass=*)"
                  attributes="#ldapRoleNameAttr#"
                  name="thisGroup">
               <cfset roleName = StructFind(thisGroup,ldapRoleNameAttr)>
               <cfset roles[i] = roleName>
      
               <cfset i = i + 1>
            </cfloop>
            <cfset rolelist = ArrayToList(roles)>
            
            <cfset user.userID = arguments.username>
            <cfset user.roles = rolelist>
            <cfreturn true>
            <cfcatch type="any">
               <cfreturn false>
            </cfcatch>
         </cftry>
   </cffunction>

   <cffunction name="setDomain" output="false" returntype="void">
      <cfargument name="domain" type="string" required="true">
      <cfset domain = arguments.domain>
   </cffunction>
   
   <cffunction name="setServer" output="false" returntype="void">
      <cfargument name="server" type="string" required="true">
      <cfset ldapServer = arguments.server>
   </cffunction>
   
   <cffunction name="setBaseDN" output="false" returntype="void">
      <cfargument name="baseDN" type="string" required="true">
      <cfset ldapBaseDN = arguments.basedN>
   </cffunction>
</cfcomponent>
