import java.io.StringReader;
import java.io.IOException;
import org.xml.sax.InputSource;
import com.allaire.wddx.WddxDeserializer;
import com.allaire.wddx.WddxDeserializationException;
import com.allaire.util.RecordSet;
import java.io.FileInputStream;
import java.util.Dictionary;

public class WDDXDeserializerExample
{
   public static final void main(String[] args)
      throws Exception
   {
      FileInputStream fin = 
         new FileInputStream("c:/skiConditions.wddx");
      InputSource in = new InputSource(fin);
      Object obj = deserializeWDDX(in);
        
      if (!(obj instanceof Dictionary)) {
         throw new Exception("obj not a dictionary");
      }
      Dictionary ski = (Dictionary)obj;
      
      System.out.println("Temp: " 
         + ski.get("CURRENTTEMPERATURE"));
       
      System.out.println("New Snow: "
         + ski.get("NEWSNOW"));
        
      System.out.println("Forecast: "
         + ski.get("WEATHERFORECAST"));
        
      System.out.println("Avg Base: "
         + ski.get("AVERAGESNOWBASE"));
        
      Object trailObj = ski.get("TRAILS");
      if (! (trailObj instanceof RecordSet) ) {
         throw new Exception("trails is not a query");
      }
      RecordSet trails = (RecordSet)trailObj;
      int rows = trails.getRowCount();
      int name = trails.findColumn("name");
      int isOpen = trails.findColumn("isOpen");
      int skill = trails.findColumn("skillLevel");
        
      System.out.println("Trails:");
      StringBuffer sb = new StringBuffer(32);
      for (int row=0;row<rows;row++) {
         sb.setLength(0);
         sb.append(trails.getField(row, name));
         sb.append(" - open: ");
         sb.append(trails.getField(row, isOpen));
         sb.append(" - skill level: ");
         sb.append(trails.getField(row, skill));
         System.out.println(sb.toString());          
      }
   }
    
    
   public static Object deserializeWDDX(String wddxPacket) 
      throws WddxDeserializationException, IOException
   {
      InputSource in = 
         new InputSource(new StringReader(wddxPacket));
      return deserializeWDDX(in);
   }
    
   public static Object deserializeWDDX(InputSource in)
      throws WddxDeserializationException, IOException
   {
      WddxDeserializer deserializer = 
        new WddxDeserializer("org.apache.xerces.parsers.SAXParser");
        
      // Deserialize the WDDX packet
      return deserializer.deserialize(in);
   }
    
}
