
package com.allaire.cfx ;

/**
*  Interface for implementing custom tags.
*
*  <P>Classes that implement this interface can be specified in the
*  <code>CLASS</code> attribute of the <code>&lt;CFX_J&gt;</code> tag. For example,
*  if I have a class <code>MyCustomTag</code> which implements this
*  interface then the following CFML code could be used to call the
*  <code>MyCustomTag.processRequest</code> method: </P>
*
*  <pre><code>   &lt;CFX_J CLASS=&quot;MyCustomTag&quot;&gt; </code></pre> 
*
*  <P>Additional attributes may also be passed to the <code>&lt;CFX_J&gt;</code> 
*  tag. The values of these attributes are available via the <code>{@link Request}</code>
*  object passed to the <code>processRequest</code> method.</P>
*
*  @author J.J. Allaire
*/
public interface CustomTag 
{
	/**
	* Processes a request originating from the <code>&lt;CFX_J&gt;</code> tag. <P>
	*
	* @param	request parameters (attributes, query, etc.) for this request
	* @param    response interface for generating response to request (output, variables, queries, etc.)
	*
	* @throws <code>Exception</code> If an unexpected error occurs while processing the request.
	*/
	public void processRequest( Request request, Response response ) throws Exception ;
}

