
package com.allaire.cfx ;

import java.util.Enumeration ;
import java.util.Hashtable ;
import java.util.Vector ;


/**
 * a debugging implementation of the com.allaire.cfx.Response interface
 */
public class DebugResponse implements Response
{
	// Initialize a debug response 
	public DebugResponse() 
	{       
      // initialize dynamic output with header
      write( "\n[Output]\n" ) ;    
   }
  
   // print the results of processing this request
   public void printResults()
   {   
      // append results of request to output buffer
      
      // debug output
      write( "\n[Debug Output]\n" ) ;
      write( m_debugOutput.toString() ) ;
      
      // variables set
      writeHashtable( "Variables Set", m_variables ) ;
      
      // queries created
      Enumeration e = m_queries.elements() ;
      while ( e.hasMoreElements() )
         writeQuery( (Query) e.nextElement() ) ;
   
      // write results to stdout
      System.out.print( m_output ) ;      
   }   
		

   // write output back to the client
	public void write( String output ) 
	{
	   if ( output != null )
	      m_output.append(output+"\n") ; 
	}

   
   // set a variable within the request scope
	public void setVariable( String name, String value ) throws IllegalArgumentException 
	{
      // check for a null name 
      if ( name == null ) 
      {
         throw new IllegalArgumentException( 
            "null passed as name argument to setVariable method" ) ;
      }
      
      // Convert null value into empty string
      String strValue = value ;
      if ( strValue == null )
         strValue = "" ;
         
      // set the variable
      m_variables.put(name, strValue) ; 
	}

      
   // add a query into the request scope and return it to the caller
	public Query addQuery( String name, String[] columns ) throws IllegalArgumentException 
	{
      // create the query
      DebugQuery query = new DebugQuery(name, columns) ;
      
      // add it to our map of queries
      m_queries.addElement(query) ;
      
      // return it to the caller
      return query ;      
	}


   // write output to our debug stream
	public void writeDebug( String output ) 
	{
	   if ( output != null )
	      m_debugOutput.append(output+"\n") ;
	}
	

	// helper function used to print a hashtable
	private void writeHashtable( String strName, Hashtable table )
	{
	   // write header
	   write ( "\n[" + strName + "]\n" ) ;
	   
	   // write values
	   Enumeration e = table.keys() ;
	   while ( e.hasMoreElements() )
	   {
	      String strKey = (String) e.nextElement() ;
	      String strValue = (String) table.get(strKey) ;
	      write( strKey + "=" + strValue ) ;    
	   }	
   }
	
	
	// helper function used to print a query
	private void writeQuery( Query query ) 
	{
		// write header
		write( "\n[Query: " + query.getName()+ "]\n" ) ;	

		// write column names
		String strColumns = "" ;
		String[] columnsOut = query.getColumns() ;
		int nColumnCount = columnsOut.length ;
		String strPrefix = "" ;
		for ( int i=0; i<nColumnCount; i++ )
		{
		   strColumns =  strColumns + strPrefix + columnsOut[i] ;
		   if ( strPrefix.length() == 0 )
		      strPrefix = ", " ;			
		}
		write( strColumns + "\n" ) ;

		// write data
		int nRowCount = query.getRowCount() ;
		for ( int iRow=1; iRow <= nRowCount; iRow++ )
		{
   		StringBuffer buffRow = new StringBuffer() ;
		   strPrefix = "" ;			
			for ( int iCol=1; iCol <= nColumnCount; iCol++ )
      	{
				buffRow.append( strPrefix + query.getData( iRow, iCol ) ) ;
				if ( strPrefix.length() == 0 )
				   strPrefix = ", " ;		   	   
			}
			write( buffRow.toString() ) ;
		}
	}

	
   // implementation data
   Hashtable m_variables = new Hashtable() ;
   Vector m_queries = new Vector() ;
   StringBuffer m_output = new StringBuffer() ;
   StringBuffer m_debugOutput = new StringBuffer() ;
}


